#ifndef QT_WAYLAND_SERVER_PLASMA_EFFECTS
#define QT_WAYLAND_SERVER_PLASMA_EFFECTS

#include "wayland-server.h"
#include "wayland-plasma-effects-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  org_kde_plasma_effects
    {
    public:
        org_kde_plasma_effects(struct ::wl_client *client, int id, int version);
        org_kde_plasma_effects(struct ::wl_display *display, int version);
        org_kde_plasma_effects();

        virtual ~org_kde_plasma_effects();

        class Resource
        {
        public:
            Resource() : effects_object(0), handle(0) {}
            virtual ~Resource() {}

            org_kde_plasma_effects *effects_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        enum location {
            location_none = 0,
            location_left = 1,
            location_top = 2,
            location_right = 3,
            location_bottom = 4
        };

    protected:
        virtual Resource *effects_allocate();

        virtual void effects_bind_resource(Resource *resource);
        virtual void effects_destroy_resource(Resource *resource);

        virtual void effects_slide(Resource *resource, struct ::wl_resource *output, struct ::wl_resource *surface, uint32_t from, int32_t x, int32_t y);
        virtual void effects_set_blur_behind_region(Resource *resource, struct ::wl_resource *surface, struct ::wl_resource *region);
        virtual void effects_set_contrast_region(Resource *resource, struct ::wl_resource *surface, struct ::wl_resource *region, uint32_t contrast, uint32_t intensity, uint32_t saturation);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::org_kde_plasma_effects_interface m_org_kde_plasma_effects_interface;

        static void handle_slide(
            ::wl_client *client,
            struct wl_resource *resource,
            struct ::wl_resource *output,
            struct ::wl_resource *surface,
            uint32_t from,
            int32_t x,
            int32_t y);
        static void handle_set_blur_behind_region(
            ::wl_client *client,
            struct wl_resource *resource,
            struct ::wl_resource *surface,
            struct ::wl_resource *region);
        static void handle_set_contrast_region(
            ::wl_client *client,
            struct wl_resource *resource,
            struct ::wl_resource *surface,
            struct ::wl_resource *region,
            uint32_t contrast,
            uint32_t intensity,
            uint32_t saturation);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
