#ifndef QT_WAYLAND_SERVER_PLASMA_SHELL
#define QT_WAYLAND_SERVER_PLASMA_SHELL

#include "wayland-server.h"
#include "wayland-plasma-shell-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  org_kde_plasma_shell
    {
    public:
        org_kde_plasma_shell(struct ::wl_client *client, int id, int version);
        org_kde_plasma_shell(struct ::wl_display *display, int version);
        org_kde_plasma_shell();

        virtual ~org_kde_plasma_shell();

        class Resource
        {
        public:
            Resource() : shell_object(0), handle(0) {}
            virtual ~Resource() {}

            org_kde_plasma_shell *shell_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        void send_loaded();
        void send_loaded(struct ::wl_resource *resource);

    protected:
        virtual Resource *shell_allocate();

        virtual void shell_bind_resource(Resource *resource);
        virtual void shell_destroy_resource(Resource *resource);

        virtual void shell_get_surface(Resource *resource, uint32_t id, struct ::wl_resource *surface);
        virtual void shell_desktop_ready(Resource *resource);
        virtual void shell_present(Resource *resource, wl_array *surfaces);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::org_kde_plasma_shell_interface m_org_kde_plasma_shell_interface;

        static void handle_get_surface(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t id,
            struct ::wl_resource *surface);
        static void handle_desktop_ready(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_present(
            ::wl_client *client,
            struct wl_resource *resource,
            wl_array *surfaces);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };

    class  org_kde_plasma_surface
    {
    public:
        org_kde_plasma_surface(struct ::wl_client *client, int id, int version);
        org_kde_plasma_surface(struct ::wl_display *display, int version);
        org_kde_plasma_surface();

        virtual ~org_kde_plasma_surface();

        class Resource
        {
        public:
            Resource() : surface_object(0), handle(0) {}
            virtual ~Resource() {}

            org_kde_plasma_surface *surface_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        enum role {
            role_none = 0,
            role_splash = 1,
            role_desktop = 2,
            role_dashboard = 3,
            role_panel = 4,
            role_overlay = 5,
            role_notification = 6,
            role_lock = 7
        };

        enum flag {
            flag_panel_always_visible = 1, // normal panel visibility
            flag_panel_auto_hide = 2, // hide automatically
            flag_panel_windows_can_cover = 3, // windows can cover
            flag_panel_windows_go_below = 4 // windows go below
        };

        enum screen_edge {
            screen_edge_none = 0,
            screen_edge_left = 1,
            screen_edge_top = 2,
            screen_edge_right = 3,
            screen_edge_bottom = 4
        };

    protected:
        virtual Resource *surface_allocate();

        virtual void surface_bind_resource(Resource *resource);
        virtual void surface_destroy_resource(Resource *resource);

        virtual void surface_destroy(Resource *resource);
        virtual void surface_set_output(Resource *resource, struct ::wl_resource *output);
        virtual void surface_set_position(Resource *resource, int32_t x, int32_t y);
        virtual void surface_set_role(Resource *resource, uint32_t role);
        virtual void surface_set_flags(Resource *resource, uint32_t flags);
        virtual void surface_set_screen_edge(Resource *resource, uint32_t screen_edge);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::org_kde_plasma_surface_interface m_org_kde_plasma_surface_interface;

        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_output(
            ::wl_client *client,
            struct wl_resource *resource,
            struct ::wl_resource *output);
        static void handle_set_position(
            ::wl_client *client,
            struct wl_resource *resource,
            int32_t x,
            int32_t y);
        static void handle_set_role(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t role);
        static void handle_set_flags(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t flags);
        static void handle_set_screen_edge(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t screen_edge);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
