/* 
 * Copyright (C) 2014 Pier Luigi Fiorini
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PLASMA_EFFECTS_SERVER_PROTOCOL_H
#define PLASMA_EFFECTS_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct org_kde_plasma_effects;
struct wl_output;
struct wl_region;
struct wl_surface;

extern const struct wl_interface org_kde_plasma_effects_interface;

#ifndef ORG_KDE_PLASMA_EFFECTS_LOCATION_ENUM
#define ORG_KDE_PLASMA_EFFECTS_LOCATION_ENUM
enum org_kde_plasma_effects_location {
	ORG_KDE_PLASMA_EFFECTS_LOCATION_NONE = 0,
	ORG_KDE_PLASMA_EFFECTS_LOCATION_LEFT = 1,
	ORG_KDE_PLASMA_EFFECTS_LOCATION_TOP = 2,
	ORG_KDE_PLASMA_EFFECTS_LOCATION_RIGHT = 3,
	ORG_KDE_PLASMA_EFFECTS_LOCATION_BOTTOM = 4,
};
#endif /* ORG_KDE_PLASMA_EFFECTS_LOCATION_ENUM */

struct org_kde_plasma_effects_interface {
	/**
	 * slide - slide a surface from a location to another
	 * @output: (none)
	 * @surface: (none)
	 * @from: (none)
	 * @x: (none)
	 * @y: (none)
	 *
	 * Ask the compositor to move the surface from a location to
	 * another with a slide animation.
	 *
	 * The from argument provides a clue about where the slide
	 * animation begins, destination coordinates are specified with x
	 * and y.
	 */
	void (*slide)(struct wl_client *client,
		      struct wl_resource *resource,
		      struct wl_resource *output,
		      struct wl_resource *surface,
		      uint32_t from,
		      int32_t x,
		      int32_t y);
	/**
	 * set_blur_behind_region - set blur behind region
	 * @surface: (none)
	 * @region: (none)
	 *
	 * This request sets the region of the surface that will allow to
	 * see through with a blur effect.
	 *
	 * Pass a null region to disable blur behind.
	 */
	void (*set_blur_behind_region)(struct wl_client *client,
				       struct wl_resource *resource,
				       struct wl_resource *surface,
				       struct wl_resource *region);
	/**
	 * set_contrast_region - set contrast region
	 * @surface: (none)
	 * @region: (none)
	 * @contrast: (none)
	 * @intensity: (none)
	 * @saturation: (none)
	 *
	 * This request sets the region of the surface with a different
	 * contrast.
	 *
	 * Pass a null region to disable this effect.
	 *
	 * When a null region is passed the contrast, intensity and
	 * saturation arguments are not taken into account.
	 *
	 * The contrast, intensity and saturation parameters are in the
	 * 0-255 range.
	 */
	void (*set_contrast_region)(struct wl_client *client,
				    struct wl_resource *resource,
				    struct wl_resource *surface,
				    struct wl_resource *region,
				    uint32_t contrast,
				    uint32_t intensity,
				    uint32_t saturation);
};


#ifdef  __cplusplus
}
#endif

#endif
