#include "qwayland-server-greenisland-recorder.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    greenisland_recorder_manager::greenisland_recorder_manager(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    greenisland_recorder_manager::greenisland_recorder_manager(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    greenisland_recorder_manager::greenisland_recorder_manager()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    greenisland_recorder_manager::~greenisland_recorder_manager()
    {
    }

    void greenisland_recorder_manager::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    greenisland_recorder_manager::Resource *greenisland_recorder_manager::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    greenisland_recorder_manager::Resource *greenisland_recorder_manager::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void greenisland_recorder_manager::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::greenisland_recorder_manager_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    greenisland_recorder_manager::Resource *greenisland_recorder_manager::recorder_manager_allocate()
    {
        return new Resource;
    }

    void greenisland_recorder_manager::recorder_manager_bind_resource(Resource *)
    {
    }

    void greenisland_recorder_manager::recorder_manager_destroy_resource(Resource *)
    {
    }

    void greenisland_recorder_manager::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        greenisland_recorder_manager *that = static_cast<greenisland_recorder_manager *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void greenisland_recorder_manager::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        greenisland_recorder_manager *that = resource->recorder_manager_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->recorder_manager_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    greenisland_recorder_manager::Resource *greenisland_recorder_manager::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = recorder_manager_allocate();
        resource->recorder_manager_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::greenisland_recorder_manager_interface, version, id);
        wl_resource_set_implementation(handle, &m_greenisland_recorder_manager_interface, resource, destroy_func);
        resource->handle = handle;
        recorder_manager_bind_resource(resource);
        return resource;
    }

    const struct ::greenisland_recorder_manager_interface greenisland_recorder_manager::m_greenisland_recorder_manager_interface = {
        greenisland_recorder_manager::handle_create_recorder
    };

    void greenisland_recorder_manager::recorder_manager_create_recorder(Resource *, uint32_t, struct ::wl_resource *)
    {
    }


    void greenisland_recorder_manager::handle_create_recorder(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t recorder,
        struct ::wl_resource *output)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_recorder_manager *>(r->recorder_manager_object)->recorder_manager_create_recorder(
            r,
            recorder,
            output);
    }

    greenisland_recorder::greenisland_recorder(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    greenisland_recorder::greenisland_recorder(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    greenisland_recorder::greenisland_recorder()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    greenisland_recorder::~greenisland_recorder()
    {
    }

    void greenisland_recorder::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    greenisland_recorder::Resource *greenisland_recorder::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    greenisland_recorder::Resource *greenisland_recorder::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void greenisland_recorder::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::greenisland_recorder_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    greenisland_recorder::Resource *greenisland_recorder::recorder_allocate()
    {
        return new Resource;
    }

    void greenisland_recorder::recorder_bind_resource(Resource *)
    {
    }

    void greenisland_recorder::recorder_destroy_resource(Resource *)
    {
    }

    void greenisland_recorder::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        greenisland_recorder *that = static_cast<greenisland_recorder *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void greenisland_recorder::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        greenisland_recorder *that = resource->recorder_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->recorder_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    greenisland_recorder::Resource *greenisland_recorder::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = recorder_allocate();
        resource->recorder_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::greenisland_recorder_interface, version, id);
        wl_resource_set_implementation(handle, &m_greenisland_recorder_interface, resource, destroy_func);
        resource->handle = handle;
        recorder_bind_resource(resource);
        return resource;
    }

    const struct ::greenisland_recorder_interface greenisland_recorder::m_greenisland_recorder_interface = {
        greenisland_recorder::handle_destroy,
        greenisland_recorder::handle_record_frame,
        greenisland_recorder::handle_repaint
    };

    void greenisland_recorder::recorder_destroy(Resource *)
    {
    }

    void greenisland_recorder::recorder_record_frame(Resource *, struct ::wl_resource *)
    {
    }

    void greenisland_recorder::recorder_repaint(Resource *)
    {
    }


    void greenisland_recorder::handle_destroy(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_recorder *>(r->recorder_object)->recorder_destroy(
            r);
    }

    void greenisland_recorder::handle_record_frame(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *buffer)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_recorder *>(r->recorder_object)->recorder_record_frame(
            r,
            buffer);
    }

    void greenisland_recorder::handle_repaint(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_recorder *>(r->recorder_object)->recorder_repaint(
            r);
    }

    void greenisland_recorder::send_setup(int32_t width, int32_t height, int32_t stride, int32_t format)
    {
        send_setup(
            m_resource->handle,
            width,
            height,
            stride,
            format);
    }

    void greenisland_recorder::send_setup(struct ::wl_resource *resource, int32_t width, int32_t height, int32_t stride, int32_t format)
    {
        greenisland_recorder_send_setup(
            resource,
            width,
            height,
            stride,
            format);
    }


    void greenisland_recorder::send_frame(struct ::wl_resource *buffer, uint32_t time, int32_t transform)
    {
        send_frame(
            m_resource->handle,
            buffer,
            time,
            transform);
    }

    void greenisland_recorder::send_frame(struct ::wl_resource *resource, struct ::wl_resource *buffer, uint32_t time, int32_t transform)
    {
        greenisland_recorder_send_frame(
            resource,
            buffer,
            time,
            transform);
    }


    void greenisland_recorder::send_failed(int32_t result, struct ::wl_resource *buffer)
    {
        send_failed(
            m_resource->handle,
            result,
            buffer);
    }

    void greenisland_recorder::send_failed(struct ::wl_resource *resource, int32_t result, struct ::wl_resource *buffer)
    {
        greenisland_recorder_send_failed(
            resource,
            result,
            buffer);
    }


    void greenisland_recorder::send_cancelled(struct ::wl_resource *buffer)
    {
        send_cancelled(
            m_resource->handle,
            buffer);
    }

    void greenisland_recorder::send_cancelled(struct ::wl_resource *resource, struct ::wl_resource *buffer)
    {
        greenisland_recorder_send_cancelled(
            resource,
            buffer);
    }

}

QT_END_NAMESPACE
