#ifndef QT_WAYLAND_SERVER_GREENISLAND_RECORDER
#define QT_WAYLAND_SERVER_GREENISLAND_RECORDER

#include "wayland-server.h"
#include "wayland-greenisland-recorder-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  greenisland_recorder_manager
    {
    public:
        greenisland_recorder_manager(struct ::wl_client *client, int id, int version);
        greenisland_recorder_manager(struct ::wl_display *display, int version);
        greenisland_recorder_manager();

        virtual ~greenisland_recorder_manager();

        class Resource
        {
        public:
            Resource() : recorder_manager_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_recorder_manager *recorder_manager_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

    protected:
        virtual Resource *recorder_manager_allocate();

        virtual void recorder_manager_bind_resource(Resource *resource);
        virtual void recorder_manager_destroy_resource(Resource *resource);

        virtual void recorder_manager_create_recorder(Resource *resource, uint32_t recorder, struct ::wl_resource *output);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::greenisland_recorder_manager_interface m_greenisland_recorder_manager_interface;

        static void handle_create_recorder(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t recorder,
            struct ::wl_resource *output);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };

    class  greenisland_recorder
    {
    public:
        greenisland_recorder(struct ::wl_client *client, int id, int version);
        greenisland_recorder(struct ::wl_display *display, int version);
        greenisland_recorder();

        virtual ~greenisland_recorder();

        class Resource
        {
        public:
            Resource() : recorder_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_recorder *recorder_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        enum result {
            result_bad_buffer = 2
        };

        enum transform {
            transform_normal = 1,
            transform_y_inverted = 2
        };

        void send_setup(int32_t width, int32_t height, int32_t stride, int32_t format);
        void send_setup(struct ::wl_resource *resource, int32_t width, int32_t height, int32_t stride, int32_t format);
        void send_frame(struct ::wl_resource *buffer, uint32_t time, int32_t transform);
        void send_frame(struct ::wl_resource *resource, struct ::wl_resource *buffer, uint32_t time, int32_t transform);
        void send_failed(int32_t result, struct ::wl_resource *buffer);
        void send_failed(struct ::wl_resource *resource, int32_t result, struct ::wl_resource *buffer);
        void send_cancelled(struct ::wl_resource *buffer);
        void send_cancelled(struct ::wl_resource *resource, struct ::wl_resource *buffer);

    protected:
        virtual Resource *recorder_allocate();

        virtual void recorder_bind_resource(Resource *resource);
        virtual void recorder_destroy_resource(Resource *resource);

        virtual void recorder_destroy(Resource *resource);
        virtual void recorder_record_frame(Resource *resource, struct ::wl_resource *buffer);
        virtual void recorder_repaint(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::greenisland_recorder_interface m_greenisland_recorder_interface;

        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_record_frame(
            ::wl_client *client,
            struct wl_resource *resource,
            struct ::wl_resource *buffer);
        static void handle_repaint(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
