#ifndef QT_WAYLAND_SERVER_GREENISLAND_SCREENSHOOTER
#define QT_WAYLAND_SERVER_GREENISLAND_SCREENSHOOTER

#include "wayland-server.h"
#include "wayland-greenisland-screenshooter-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  greenisland_screenshooter
    {
    public:
        greenisland_screenshooter(struct ::wl_client *client, int id, int version);
        greenisland_screenshooter(struct ::wl_display *display, int version);
        greenisland_screenshooter();

        virtual ~greenisland_screenshooter();

        class Resource
        {
        public:
            Resource() : screenshooter_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_screenshooter *screenshooter_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        enum result {
            result_bad_output = 1,
            result_bad_buffer = 2,
            result_success = 3
        };

        void send_done(int32_t result);
        void send_done(struct ::wl_resource *resource, int32_t result);

    protected:
        virtual Resource *screenshooter_allocate();

        virtual void screenshooter_bind_resource(Resource *resource);
        virtual void screenshooter_destroy_resource(Resource *resource);

        virtual void screenshooter_shoot(Resource *resource, struct ::wl_resource *output, struct ::wl_resource *buffer);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::greenisland_screenshooter_interface m_greenisland_screenshooter_interface;

        static void handle_shoot(
            ::wl_client *client,
            struct wl_resource *resource,
            struct ::wl_resource *output,
            struct ::wl_resource *buffer);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
