#include "qwayland-server-greenisland.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    greenisland_applications::greenisland_applications(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    greenisland_applications::greenisland_applications(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    greenisland_applications::greenisland_applications()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    greenisland_applications::~greenisland_applications()
    {
    }

    void greenisland_applications::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    greenisland_applications::Resource *greenisland_applications::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    greenisland_applications::Resource *greenisland_applications::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void greenisland_applications::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::greenisland_applications_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    greenisland_applications::Resource *greenisland_applications::applications_allocate()
    {
        return new Resource;
    }

    void greenisland_applications::applications_bind_resource(Resource *)
    {
    }

    void greenisland_applications::applications_destroy_resource(Resource *)
    {
    }

    void greenisland_applications::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        greenisland_applications *that = static_cast<greenisland_applications *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void greenisland_applications::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        greenisland_applications *that = resource->applications_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->applications_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    greenisland_applications::Resource *greenisland_applications::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = applications_allocate();
        resource->applications_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::greenisland_applications_interface, version, id);
        wl_resource_set_implementation(handle, &m_greenisland_applications_interface, resource, destroy_func);
        resource->handle = handle;
        applications_bind_resource(resource);
        return resource;
    }

    const struct ::greenisland_applications_interface greenisland_applications::m_greenisland_applications_interface = {
        greenisland_applications::handle_quit
    };

    void greenisland_applications::applications_quit(Resource *, const QString &)
    {
    }


    void greenisland_applications::handle_quit(
        ::wl_client *client,
        struct wl_resource *resource,
        const char *app_id)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<greenisland_applications *>(r->applications_object)->applications_quit(
            r,
            QString::fromUtf8(app_id));
    }

    void greenisland_applications::send_registered(const QString &app_id, uint32_t pid)
    {
        send_registered(
            m_resource->handle,
            app_id,
            pid);
    }

    void greenisland_applications::send_registered(struct ::wl_resource *resource, const QString &app_id, uint32_t pid)
    {
        greenisland_applications_send_registered(
            resource,
            app_id.toUtf8().constData(),
            pid);
    }


    void greenisland_applications::send_unregistered(const QString &app_id, uint32_t pid)
    {
        send_unregistered(
            m_resource->handle,
            app_id,
            pid);
    }

    void greenisland_applications::send_unregistered(struct ::wl_resource *resource, const QString &app_id, uint32_t pid)
    {
        greenisland_applications_send_unregistered(
            resource,
            app_id.toUtf8().constData(),
            pid);
    }


    void greenisland_applications::send_focused(const QString &app_id)
    {
        send_focused(
            m_resource->handle,
            app_id);
    }

    void greenisland_applications::send_focused(struct ::wl_resource *resource, const QString &app_id)
    {
        greenisland_applications_send_focused(
            resource,
            app_id.toUtf8().constData());
    }


    void greenisland_applications::send_unfocused(const QString &app_id)
    {
        send_unfocused(
            m_resource->handle,
            app_id);
    }

    void greenisland_applications::send_unfocused(struct ::wl_resource *resource, const QString &app_id)
    {
        greenisland_applications_send_unfocused(
            resource,
            app_id.toUtf8().constData());
    }


    greenisland_windows::greenisland_windows(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    greenisland_windows::greenisland_windows(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    greenisland_windows::greenisland_windows()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    greenisland_windows::~greenisland_windows()
    {
    }

    void greenisland_windows::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    greenisland_windows::Resource *greenisland_windows::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    greenisland_windows::Resource *greenisland_windows::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void greenisland_windows::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::greenisland_windows_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    greenisland_windows::Resource *greenisland_windows::windows_allocate()
    {
        return new Resource;
    }

    void greenisland_windows::windows_bind_resource(Resource *)
    {
    }

    void greenisland_windows::windows_destroy_resource(Resource *)
    {
    }

    void greenisland_windows::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        greenisland_windows *that = static_cast<greenisland_windows *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void greenisland_windows::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        greenisland_windows *that = resource->windows_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->windows_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    greenisland_windows::Resource *greenisland_windows::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = windows_allocate();
        resource->windows_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::greenisland_windows_interface, version, id);
        wl_resource_set_implementation(handle, 0, resource, destroy_func);
        resource->handle = handle;
        windows_bind_resource(resource);
        return resource;
    }

    void greenisland_windows::send_window_mapped(struct ::wl_resource *id, uint32_t type, uint32_t state, const QString &title, const QString &app_id)
    {
        send_window_mapped(
            m_resource->handle,
            id,
            type,
            state,
            title,
            app_id);
    }

    void greenisland_windows::send_window_mapped(struct ::wl_resource *resource, struct ::wl_resource *id, uint32_t type, uint32_t state, const QString &title, const QString &app_id)
    {
        greenisland_windows_send_window_mapped(
            resource,
            id,
            type,
            state,
            title.toUtf8().constData(),
            app_id.toUtf8().constData());
    }


    greenisland_window::greenisland_window(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    greenisland_window::greenisland_window(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    greenisland_window::greenisland_window()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    greenisland_window::~greenisland_window()
    {
    }

    void greenisland_window::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    greenisland_window::Resource *greenisland_window::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    greenisland_window::Resource *greenisland_window::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void greenisland_window::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::greenisland_window_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    greenisland_window::Resource *greenisland_window::window_allocate()
    {
        return new Resource;
    }

    void greenisland_window::window_bind_resource(Resource *)
    {
    }

    void greenisland_window::window_destroy_resource(Resource *)
    {
    }

    void greenisland_window::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        greenisland_window *that = static_cast<greenisland_window *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void greenisland_window::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        greenisland_window *that = resource->window_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->window_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    greenisland_window::Resource *greenisland_window::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = window_allocate();
        resource->window_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::greenisland_window_interface, version, id);
        wl_resource_set_implementation(handle, 0, resource, destroy_func);
        resource->handle = handle;
        window_bind_resource(resource);
        return resource;
    }

    void greenisland_window::send_title_changed(const QString &title)
    {
        send_title_changed(
            m_resource->handle,
            title);
    }

    void greenisland_window::send_title_changed(struct ::wl_resource *resource, const QString &title)
    {
        greenisland_window_send_title_changed(
            resource,
            title.toUtf8().constData());
    }


    void greenisland_window::send_app_id_changed(const QString &app_id)
    {
        send_app_id_changed(
            m_resource->handle,
            app_id);
    }

    void greenisland_window::send_app_id_changed(struct ::wl_resource *resource, const QString &app_id)
    {
        greenisland_window_send_app_id_changed(
            resource,
            app_id.toUtf8().constData());
    }


    void greenisland_window::send_state_changed(uint32_t flags)
    {
        send_state_changed(
            m_resource->handle,
            flags);
    }

    void greenisland_window::send_state_changed(struct ::wl_resource *resource, uint32_t flags)
    {
        greenisland_window_send_state_changed(
            resource,
            flags);
    }


    void greenisland_window::send_workspace_changed(uint32_t number)
    {
        send_workspace_changed(
            m_resource->handle,
            number);
    }

    void greenisland_window::send_workspace_changed(struct ::wl_resource *resource, uint32_t number)
    {
        greenisland_window_send_workspace_changed(
            resource,
            number);
    }


    void greenisland_window::send_unmapped()
    {
        send_unmapped(
            m_resource->handle);
    }

    void greenisland_window::send_unmapped(struct ::wl_resource *resource)
    {
        greenisland_window_send_unmapped(
            resource);
    }

}

QT_END_NAMESPACE
