#ifndef QT_WAYLAND_SERVER_GREENISLAND
#define QT_WAYLAND_SERVER_GREENISLAND

#include "wayland-server.h"
#include "wayland-greenisland-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  greenisland_applications
    {
    public:
        greenisland_applications(struct ::wl_client *client, int id, int version);
        greenisland_applications(struct ::wl_display *display, int version);
        greenisland_applications();

        virtual ~greenisland_applications();

        class Resource
        {
        public:
            Resource() : applications_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_applications *applications_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        void send_registered(const QString &app_id, uint32_t pid);
        void send_registered(struct ::wl_resource *resource, const QString &app_id, uint32_t pid);
        void send_unregistered(const QString &app_id, uint32_t pid);
        void send_unregistered(struct ::wl_resource *resource, const QString &app_id, uint32_t pid);
        void send_focused(const QString &app_id);
        void send_focused(struct ::wl_resource *resource, const QString &app_id);
        void send_unfocused(const QString &app_id);
        void send_unfocused(struct ::wl_resource *resource, const QString &app_id);

    protected:
        virtual Resource *applications_allocate();

        virtual void applications_bind_resource(Resource *resource);
        virtual void applications_destroy_resource(Resource *resource);

        virtual void applications_quit(Resource *resource, const QString &app_id);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::greenisland_applications_interface m_greenisland_applications_interface;

        static void handle_quit(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *app_id);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };

    class  greenisland_windows
    {
    public:
        greenisland_windows(struct ::wl_client *client, int id, int version);
        greenisland_windows(struct ::wl_display *display, int version);
        greenisland_windows();

        virtual ~greenisland_windows();

        class Resource
        {
        public:
            Resource() : windows_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_windows *windows_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        enum type {
            type_toplevel = 1,
            type_popup = 2,
            type_transient = 3
        };

        enum state {
            state_inactive = 0x0,
            state_active = 0x1,
            state_minimized = 0x2,
            state_maximized = 0x3,
            state_fullscreen = 0x4,
            state_keep_above = 0x5,
            state_keep_below = 0x6,
            state_shaded = 0x7,
            state_on_all_workspaces = 0x8
        };

        void send_window_mapped(struct ::wl_resource *id, uint32_t type, uint32_t state, const QString &title, const QString &app_id);
        void send_window_mapped(struct ::wl_resource *resource, struct ::wl_resource *id, uint32_t type, uint32_t state, const QString &title, const QString &app_id);

    protected:
        virtual Resource *windows_allocate();

        virtual void windows_bind_resource(Resource *resource);
        virtual void windows_destroy_resource(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };

    class  greenisland_window
    {
    public:
        greenisland_window(struct ::wl_client *client, int id, int version);
        greenisland_window(struct ::wl_display *display, int version);
        greenisland_window();

        virtual ~greenisland_window();

        class Resource
        {
        public:
            Resource() : window_object(0), handle(0) {}
            virtual ~Resource() {}

            greenisland_window *window_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        void send_title_changed(const QString &title);
        void send_title_changed(struct ::wl_resource *resource, const QString &title);
        void send_app_id_changed(const QString &app_id);
        void send_app_id_changed(struct ::wl_resource *resource, const QString &app_id);
        void send_state_changed(uint32_t flags);
        void send_state_changed(struct ::wl_resource *resource, uint32_t flags);
        void send_workspace_changed(uint32_t number);
        void send_workspace_changed(struct ::wl_resource *resource, uint32_t number);
        void send_unmapped();
        void send_unmapped(struct ::wl_resource *resource);

    protected:
        virtual Resource *window_allocate();

        virtual void window_bind_resource(Resource *resource);
        virtual void window_destroy_resource(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
