#include "qwayland-server-gtk.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    gtk_shell::gtk_shell(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    gtk_shell::gtk_shell(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    gtk_shell::gtk_shell()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    gtk_shell::~gtk_shell()
    {
    }

    void gtk_shell::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    gtk_shell::Resource *gtk_shell::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    gtk_shell::Resource *gtk_shell::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void gtk_shell::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::gtk_shell_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    gtk_shell::Resource *gtk_shell::shell_allocate()
    {
        return new Resource;
    }

    void gtk_shell::shell_bind_resource(Resource *)
    {
    }

    void gtk_shell::shell_destroy_resource(Resource *)
    {
    }

    void gtk_shell::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        gtk_shell *that = static_cast<gtk_shell *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void gtk_shell::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        gtk_shell *that = resource->shell_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->shell_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    gtk_shell::Resource *gtk_shell::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = shell_allocate();
        resource->shell_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::gtk_shell_interface, version, id);
        wl_resource_set_implementation(handle, &m_gtk_shell_interface, resource, destroy_func);
        resource->handle = handle;
        shell_bind_resource(resource);
        return resource;
    }

    const struct ::gtk_shell_interface gtk_shell::m_gtk_shell_interface = {
        gtk_shell::handle_get_gtk_surface
    };

    void gtk_shell::shell_get_gtk_surface(Resource *, uint32_t, struct ::wl_resource *)
    {
    }


    void gtk_shell::handle_get_gtk_surface(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t gtk_surface,
        struct ::wl_resource *surface)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<gtk_shell *>(r->shell_object)->shell_get_gtk_surface(
            r,
            gtk_surface,
            surface);
    }

    void gtk_shell::send_capabilities(uint32_t capabilities)
    {
        send_capabilities(
            m_resource->handle,
            capabilities);
    }

    void gtk_shell::send_capabilities(struct ::wl_resource *resource, uint32_t capabilities)
    {
        gtk_shell_send_capabilities(
            resource,
            capabilities);
    }


    gtk_surface::gtk_surface(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    gtk_surface::gtk_surface(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    gtk_surface::gtk_surface()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    gtk_surface::~gtk_surface()
    {
    }

    void gtk_surface::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    gtk_surface::Resource *gtk_surface::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    gtk_surface::Resource *gtk_surface::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void gtk_surface::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::gtk_surface_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    gtk_surface::Resource *gtk_surface::surface_allocate()
    {
        return new Resource;
    }

    void gtk_surface::surface_bind_resource(Resource *)
    {
    }

    void gtk_surface::surface_destroy_resource(Resource *)
    {
    }

    void gtk_surface::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        gtk_surface *that = static_cast<gtk_surface *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void gtk_surface::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        gtk_surface *that = resource->surface_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->surface_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    gtk_surface::Resource *gtk_surface::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = surface_allocate();
        resource->surface_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::gtk_surface_interface, version, id);
        wl_resource_set_implementation(handle, &m_gtk_surface_interface, resource, destroy_func);
        resource->handle = handle;
        surface_bind_resource(resource);
        return resource;
    }

    const struct ::gtk_surface_interface gtk_surface::m_gtk_surface_interface = {
        gtk_surface::handle_set_dbus_properties
    };

    void gtk_surface::surface_set_dbus_properties(Resource *, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &)
    {
    }


    void gtk_surface::handle_set_dbus_properties(
        ::wl_client *client,
        struct wl_resource *resource,
        const char *application_id,
        const char *app_menu_path,
        const char *menubar_path,
        const char *window_object_path,
        const char *application_object_path,
        const char *unique_bus_name)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<gtk_surface *>(r->surface_object)->surface_set_dbus_properties(
            r,
            QString::fromUtf8(application_id),
            QString::fromUtf8(app_menu_path),
            QString::fromUtf8(menubar_path),
            QString::fromUtf8(window_object_path),
            QString::fromUtf8(application_object_path),
            QString::fromUtf8(unique_bus_name));
    }
}

QT_END_NAMESPACE
