/* 
 * Copyright (C) 2015 Pier Luigi Fiorini
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GREENISLAND_SCREENSHOOTER_SERVER_PROTOCOL_H
#define GREENISLAND_SCREENSHOOTER_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct greenisland_screenshooter;

extern const struct wl_interface greenisland_screenshooter_interface;

#ifndef GREENISLAND_SCREENSHOOTER_RESULT_ENUM
#define GREENISLAND_SCREENSHOOTER_RESULT_ENUM
enum greenisland_screenshooter_result {
	GREENISLAND_SCREENSHOOTER_RESULT_BAD_OUTPUT = 1,
	GREENISLAND_SCREENSHOOTER_RESULT_BAD_BUFFER = 2,
	GREENISLAND_SCREENSHOOTER_RESULT_SUCCESS = 3,
};
#endif /* GREENISLAND_SCREENSHOOTER_RESULT_ENUM */

/**
 * greenisland_screenshooter - dump the contents of an output to a buffer
 * @shoot: quit an application
 *
 * This interface is used by programs to take a screenshot of an output.
 * Output contents are dumped into a buffer that is read by the screenshot
 * application and saved as an image file. The done event is sent when the
 * buffer is filled.
 */
struct greenisland_screenshooter_interface {
	/**
	 * shoot - quit an application
	 * @output: wl_output
	 * @buffer: wl_buffer
	 *
	 * Ask the compositor to terminate a Wayland client with the
	 * specified application identifier.
	 */
	void (*shoot)(struct wl_client *client,
		      struct wl_resource *resource,
		      struct wl_resource *output,
		      struct wl_resource *buffer);
};

#define GREENISLAND_SCREENSHOOTER_DONE	0

#define GREENISLAND_SCREENSHOOTER_DONE_SINCE_VERSION	1

static inline void
greenisland_screenshooter_send_done(struct wl_resource *resource_, int32_t result)
{
	wl_resource_post_event(resource_, GREENISLAND_SCREENSHOOTER_DONE, result);
}

#ifdef  __cplusplus
}
#endif

#endif
