#ifndef QT_WAYLAND_GREENISLAND_RECORDER
#define QT_WAYLAND_GREENISLAND_RECORDER

#include "wayland-greenisland-recorder-client-protocol.h"
#include <QByteArray>
#include <QString>

QT_BEGIN_NAMESPACE

namespace QtWayland {
    class  greenisland_recorder_manager
    {
    public:
        greenisland_recorder_manager(struct ::wl_registry *registry, int id, int version);
        greenisland_recorder_manager(struct ::greenisland_recorder_manager *object);
        greenisland_recorder_manager();

        virtual ~greenisland_recorder_manager();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::greenisland_recorder_manager *object);

        struct ::greenisland_recorder_manager *object() { return m_greenisland_recorder_manager; }
        const struct ::greenisland_recorder_manager *object() const { return m_greenisland_recorder_manager; }

        bool isInitialized() const;

        struct ::greenisland_recorder *create_recorder(struct ::wl_output *output);

    private:
        struct ::greenisland_recorder_manager *m_greenisland_recorder_manager;
    };

    class  greenisland_recorder
    {
    public:
        greenisland_recorder(struct ::wl_registry *registry, int id, int version);
        greenisland_recorder(struct ::greenisland_recorder *object);
        greenisland_recorder();

        virtual ~greenisland_recorder();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::greenisland_recorder *object);

        struct ::greenisland_recorder *object() { return m_greenisland_recorder; }
        const struct ::greenisland_recorder *object() const { return m_greenisland_recorder; }

        bool isInitialized() const;

        enum result {
            result_bad_buffer = 2
        };

        enum transform {
            transform_normal = 1,
            transform_y_inverted = 2
        };

        void destroy();
        void record_frame(struct ::wl_buffer *buffer);
        void repaint();

    protected:
        virtual void recorder_setup(int32_t width, int32_t height, int32_t stride, int32_t format);
        virtual void recorder_frame(struct ::wl_buffer *buffer, uint32_t time, int32_t transform);
        virtual void recorder_failed(int32_t result, struct ::wl_buffer *buffer);
        virtual void recorder_cancelled(struct ::wl_buffer *buffer);

    private:
        void init_listener();
        static const struct greenisland_recorder_listener m_greenisland_recorder_listener;
        static void handle_setup(
            void *data,
            struct ::greenisland_recorder *object,
            int32_t width,
            int32_t height,
            int32_t stride,
            int32_t format);
        static void handle_frame(
            void *data,
            struct ::greenisland_recorder *object,
            struct ::wl_buffer *buffer,
            uint32_t time,
            int32_t transform);
        static void handle_failed(
            void *data,
            struct ::greenisland_recorder *object,
            int32_t result,
            struct ::wl_buffer *buffer);
        static void handle_cancelled(
            void *data,
            struct ::greenisland_recorder *object,
            struct ::wl_buffer *buffer);
        struct ::greenisland_recorder *m_greenisland_recorder;
    };
}

QT_END_NAMESPACE

#endif
